use std::{io, str::FromStr};

type Error = Box<dyn std::error::Error + Send + Sync>;
type Result<T> = std::result::Result<T, Error>;

#[derive(Debug)]
pub struct CapCode {
    capcode: &'static str,
    region: &'static str,
    location: &'static str,
    service: &'static str,
    description: &'static str,
}

include!(concat!(env!("OUT_DIR"), "/capcodes.rs"));

#[derive(Debug)]
struct Message {
    raw: String,
    protocol: String,
    timestamp: String,
    flags: String,
    frameid: String,
    capcodes: Vec<&'static CapCode>,
    format: String,
    message: String,
}

impl FromStr for Message {
    type Err = Error;

    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
        let mut parts = s.split('|'); //.collect::<Vec<&str>>();
        if parts.clone().count() != 7 {
            return Err("Invalid message".into());
        }

        let Some(protocol) = parts.next() else {
            return Err("Missing Protocol".into());
        };
        let Some(timestamp) = parts.next() else {
            return Err("Missing Timestamp".into());
        };
        let Some(flags) = parts.next() else {
            return Err("Missing Flags".into());
        };
        let Some(frameid) = parts.next() else {
            return Err("Missing FrameID".into());
        };
        let Some(capcodes) = parts.next() else {
            return Err("Missing Cap Codes".into());
        };
        let Some(format) = parts.next() else {
            return Err("Missing Format".into());
        };
        let Some(message) = parts.next() else {
            return Err("Missing Message".into());
        };

        let mut cc = vec![];
        let parts = capcodes.split(' ');
        for c in parts {
            let c = &c[2..];
            let Some(cap) = CAPCODES.get(c) else {
                continue;
            };
            cc.push(cap);
        }

        Ok(Self {
            raw: s.to_string(),
            protocol: protocol.to_string(),
            timestamp: timestamp.to_string(),
            flags: flags.to_string(),
            frameid: frameid.to_string(),
            capcodes: cc,
            format: format.to_string(),
            message: message.trim().to_string(),
        })
    }
}

fn main() -> Result<()> {
    let mut buffer = String::new();
    let stdin = io::stdin(); // We get `Stdin` here.

    while stdin.read_line(&mut buffer)? > 0 {
        if let Err(e) = process(&buffer) {
            eprintln!("Error: {:?}", e);
        }
        buffer.clear();
    }

    Ok(())
}

fn process(s: &str) -> Result<Message> {
    let message = Message::from_str(s)?;
    dbg!(&message);

    Ok(message)
}
